###############################################################################
## Company      : Four J's Development Tools
## Program      : fgldraw.4gl
## Date         : 15/05/1997
## Last Changes : 03/04/1998
## Version      : 1.10
##
## Title        : Allow the use of tcl/tk canvas widgets.
##
##
###############################################################################



###############################################################################
##                                  GLOBALS
###############################################################################

define lineWidth, trans, isInit smallint, anchor char(2)
define LinesAreBlack smallint

###############################################################################
##                                  FUNCTIONS
###############################################################################

################################################################################
## FUNCTION DrawInit : load the canvas.tcl into the graphical client
## Parameters :
##
## Returnings :
##
## Remarks    : Mandatory before using any of the draw function.
##
################################################################################
function DrawInit()

  define fgldir char(100),
         len    smallint,
         file   char(120)

  call fgl_init4js()

  let fgldir = fgl_getenv("FGLDIR")

  IF LinesAreBlack != 1 THEN
      LET LinesAreBlack = 0
  END IF

  IF LENGTH(fgldir) = 0 THEN
    LET fgldir="/usr/fgl2c"
  END IF

  let len = LENGTH(fgldir)
  if fgldir[len,len] = "/" then
    let file = fgldir clipped, "etc/canvas.tcl"
  else
    let file = fgldir clipped, "/etc/canvas.tcl"
  end if


  if isInit then return end if
  call fgl_uiSendSource(file)
  let isInit=1
end function


################################################################################
## FUNCTION DrawSelect : allow to select an item 
## Parameters :
##    canvas : the name of the canvas to select
##
## Returnings :
##
################################################################################
function DrawSelect(canvas)
  define canvas varchar(20)
  define command varchar(100)
#  call DrawInit()
  let command="fgl_CanvasSelect ",fgl_getpid()," ",canvas
  let anchor ="s"
  let trans=0
  call fgl_UIsendcommand(command)
end function

################################################################################
## FUNCTION DisableColorLines : Unable or disable the possibility to use 
##                              DrawFillColor to set the color of the lines
##                              Default : ColorLines = 0
## Parameters :
##    ColorLines : 0 : The lines take the color defines with DrawFillColor
##                 1 : The lines are black
##
## Returnings :
##
################################################################################
function DisableColorLines(ColorLines)
  DEFINE ColorLines SMALLINT
  IF ColorLines != 1
  THEN
    LET LinesAreBlack = 0
  ELSE
    LET LinesAreBlack = 1
  END IF
END FUNCTION

################################################################################
## FUNCTION DrawFillColor : set the fill color
## Parameters :
##    col : the name of the color for filling shapes
##
## Returnings :
##
################################################################################
function DrawFillColor(col)
  define col varchar(20)
  define command varchar(100)
  let command="fgl_CvSetFillColor ",fgl_getpid()," ",col
  call fgl_UIsendcommand(command)
end function

################################################################################
## FUNCTION DrawAnchor : define how to attach one object to another
## Parameters :
##    a :  n    Center of object's top side
##         e    Center of object's right side
##         s    Center of object's bottom side
##         w    Center of object's left side
## Returnings :
##
################################################################################
function DrawAnchor(a)
  define a char(2)
  let anchor=a
  if trans then
  case a 
  when "n" let anchor="e"
  when "e" let anchor="n"
  when "s" let anchor="w"
  when "w" let anchor="s"
  end case
  end if
end function

################################################################################
## FUNCTION DrawLineWidth : define the width of lines
## Parameters :
##     w : define the size of the pen for drawing
##
## Returnings :
##
################################################################################
function DrawLineWidth(w)
  define w smallint
  let lineWidth=w
end function

################################################################################
## FUNCTION DrawClear : clear everything
## Parameters :
##
## Returnings :
##
################################################################################
function DrawClear()
  define command varchar(100)
  let command="fgl_CvClear ",fgl_getpid()
  call fgl_uiSendCommand(command)
end function

################################################################################
## FUNCTION DrawRectangle : draw a rectangle
## Parameters :
##    x,y : position of the higher left corner of the rectangle
##    dx  : x length of the rectangle
##    dy  : y length of the rectangle
## Returnings :
##        The item number in the canvas
################################################################################
function DrawRectangle(y,x,dy,dx)
  define x,y,dx,dy integer
  define command varchar(100)
  if trans then
    let command="fgl_CvRectangle",fgl_getpid()," ",y,x,dy,dx
  else
    let command="fgl_CvRectangle",fgl_getpid()," ",x,y,dx,dy
  end if
  call fgl_UIsendcommand(command)
  return fgl_uiretrieve("drawret")
end function

################################################################################
## FUNCTION DrawOval : draw an Oval from a bounding rectangle
## Parameters :
##   x,y : position of the higher left corner of the rectangle 
##   dx  : x length of the rectangle
##   dy  : y length of the rectangle
## Returnings :
##        The item number in the canvas
##
################################################################################
function DrawOval(y,x,dy,dx)
  define x,y,dx,dy integer
  define command varchar(100)
  if trans then
    let command="fgl_CvOval",fgl_getpid()," ",y,x,dy,dx
  else
    let command="fgl_CvOval",fgl_getpid()," ",x,y,dx,dy
  end if
  call fgl_UIsendcommand(command)
  return fgl_uiretrieve("drawret")
end function
################################################################################
## FUNCTION DrawCircle : draw a Circle into a bounding square
## Parameters :
##   x,y : position of the higher left corner of the square
##   dx  : length of the square borders
##
## Returnings :
##        The item number in the canvas
##
################################################################################
function DrawCircle(y,x,dx)
  define x,y,dx integer
  define command varchar(100)
  if trans then
    let command="fgl_CvCircle",fgl_getpid()," ",y,x,dx
  else
    let command="fgl_CvCircle",fgl_getpid()," ",x,y,dx
  end if
  call fgl_UIsendcommand(command)
  return fgl_uiretrieve("drawret")
end function
################################################################################
## FUNCTION Drawline : Draw a line
## Parameters :
##    x,y   : position of the start of the line
##    dx,dy : position of the end of the line
## Returnings :
##        The item number in the canvas
##
################################################################################
function DrawLine(y,x,dy,dx)
  define x,y,dx,dy smallint
  define command varchar(100)
  if trans then
    let command="fgl_CvLine",fgl_getpid()," ",y,x,dy,dx,lineWidth,LinesAreBlack
  else
    let command="fgl_CvLine",fgl_getpid()," ",x,y,dx,dy,lineWidth,LinesAreBlack
  end if
  call fgl_UIsendcommand(command)
  return fgl_uiretrieve("drawret")
end function

################################################################################
## FUNCTION DrawText : draw a text at y,x position
## Parameters :
##    x,y : position of the begining of the text
##    t   : string 
## Returnings :
##        The item number in the canvas
##
################################################################################
function DrawText(y,x,t)
  define 
    x,y integer,
    t varchar(100),
    command varchar(100)
  if trans then
    let command="fgl_CvText",fgl_getpid()," ",y,x," \"",t,"\" ",anchor
  else
    let command="fgl_CvText",fgl_getpid()," ",x,y," \"",t,"\" ",anchor
  end if
  call fgl_UIsendcommand(command)
  return fgl_uiretrieve("drawret")
end function
################################################################################
## FUNCTION DrawArc : draw an pie slice
## Parameters :
##   x,y : position of the higher left corner of the square
##   d   : length of the square borders
##   start : angle where arc starts, in degrees
##   arc : the span of the arc, in degrees
##
## Returnings :
##        The item number in the canvas
##
################################################################################
function DrawArc(y,x,d,start,arc)
  define x,y,d,start,arc integer
  define command varchar(100)
  if trans then
    let command="fgl_CvArc",fgl_getpid()," ",y,x,d,start,arc
  else
    let command="fgl_CvArc",fgl_getpid()," ",x,y,d,start,arc
  end if
  call fgl_UIsendcommand(command)
  return fgl_uiretrieve("drawret")
end function
################################################################################
## FUNCTION DrawPolygon : Draw a filled polygon defined by at least 3 vertices
## Parameters :
##  l: list of vertices. syntax: y1 x1 y2 x2 y3 x3...
##
## Returnings :
##        The item number in the canvas
##
################################################################################
function DrawPolygon(l)
  define l varchar(100),command varchar(100)
  let command="fgl_CvPolygon ",fgl_getpid()," \"",l,"\""
  call fgl_UIsendcommand(command)
  return fgl_uiretrieve("drawret")
end function
################################################################################
## FUNCTION DrawButtonLeft : Define a key to send when the user left click
##                           on an item
## Parameters :
##    noit : item number
##    key  : the key to send when there is a left click on the itemo
##
## Returnings :
##
################################################################################
function DrawButtonLeft(noit,key)
  define command varchar(100), key varchar(30)
  define noit integer
  let command="fgl_CvBind ",fgl_getpid()," ",noit," <1> ",key
  call fgl_UIsendcommand(command)
end function
################################################################################
## FUNCTION DrawButtonRight : Define a key to send when the user left click
##                           on an item
## Parameters :
##    noit : item number
##    key  : the key to send when there is a left click on the itemo
##
## Returnings :
##
################################################################################
function DrawButtonRight(noit,key)
  define command varchar(100), key varchar(30)
  define noit integer
  let command="fgl_CvBind ",fgl_getpid()," ",noit," <3> ",key
  call fgl_UIsendcommand(command)
end function
################################################################################
## FUNCTION DrawClearButton : Remove all bindings between mouse buttons
##                           on an item
## Parameters :
##    noit : item number
##
## Returnings :
##
################################################################################
function DrawClearButton(noob)
  define command varchar(100), key varchar(30)
  define noob integer
  let command="fgl_CvBind ",fgl_getpid()," ",noob," <1> ","nokey"
  let command="fgl_CvBind ",fgl_getpid()," ",noob," <3> ","nokey"
  call fgl_UIsendcommand(command)
end function
